package extensions;

import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.Socket;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import dareka.common.CloseUtil;
import dareka.common.Logger;
import dareka.extensions.Extension2;
import dareka.extensions.ExtensionManager;
import dareka.processor.HttpHeader;
import dareka.processor.HttpRequestHeader;
import dareka.processor.Processor;
import dareka.processor.Resource;
import dareka.processor.StringResource;
import dareka.processor.impl.Cache;

/**
 * MediaInfoĂяo
 */
public class nlMediaInfo implements Extension2, Processor {
	
	public static final int REVISION = 121109;
	public static final String VER_STRING = "nlMediaInfo_"+REVISION;
	
	private static final String[] PROCESSOR_SUPPORTED_METHODS = { "GET" };
	private static final Pattern PROCESSOR_SUPPORTED_PATTERN = Pattern.compile(
			"^http://www\\.nicovideo\\.jp/cache/mediainfo\\?([a-z]{2}\\d+(low)?)");
	
	// Extension2 interface
	public void registerExtensions(ExtensionManager mgr) {
		mgr.registerProcessor(this);
	}
	
	public String getVersionString() {
		return VER_STRING;
	}
	
	// Processor Interface
	public String[] getSupportedMethods() {
		return PROCESSOR_SUPPORTED_METHODS;
	}
	
	public Pattern getSupportedURLAsPattern() {
		return PROCESSOR_SUPPORTED_PATTERN;
	}
	
	public String getSupportedURLAsString() {
		return null;
	}
	
	public Resource onRequest(HttpRequestHeader requestHeader, Socket browser)
			throws IOException {
		Matcher m = PROCESSOR_SUPPORTED_PATTERN.matcher(requestHeader.getURI());
		if (m.find()) {
			String smid = m.group(1);
			File cacheFile = Cache.getId2File().get(smid);
			if (cacheFile != null) {
				String xml = getMediaInfoXml(cacheFile);
				if (xml != null) {
					StringResource r = new StringResource(xml);
					r.addResponseHeader(HttpHeader.CONTENT_TYPE, "text/xml");
					r.addNoCacheResponseHeaders();
					return r;
				}
			}
		}
		return StringResource.getNotFound();
	}
	
	private String getMediaInfoXml(File cacheFile) {
		String[] cmdarray = { "mediainfo", "--Output=XML", cacheFile.getPath() };
		Process proc = null;
		InputStreamReader in = null;
		try {
			proc = Runtime.getRuntime().exec(cmdarray);
			in = new InputStreamReader(proc.getInputStream());
			StringBuilder sb = new StringBuilder(1024 * 8);
			char[] cbuf = new char[1024];
			for (int n; (n = in.read(cbuf)) != -1; )
				sb.append(cbuf, 0, n);
			proc.waitFor();
			return sb.toString();
		} catch (IOException e) {
			Logger.error(e);
		} catch (InterruptedException e) {
			Logger.error(e);
		} finally {
			if (proc != null) {
				CloseUtil.close(in);
				CloseUtil.close(proc.getOutputStream());
				CloseUtil.close(proc.getErrorStream());
				proc.destroy();
			}
		}
		return null;
	}
	
}
